import socket
import string
import sys
import os

total_zones = 4

default_socket_path = "/tmp/pulse-mpatterson/cli"
#default_socket_path = "/tmp/pulse-" + os.environ.get("USER") + "/cli"
pa_prompt_string = ">>>"

pa_index_string = "index: "
pa_list_sink_inputs_string = "list-sink-inputs"
pa_mute_string = "mute: <"
pa_set_sink_volume_string = "set-sink-volume"
pa_list_sinks_string = "list-sinks"
pa_volume_string = "volume: <"

pa_first_input_number = 0
pa_first_sink_number = 0
# this is the stream index numbers that feed each zone
pa_zone_inputs = ((0, 4, 8, 12), (1, 5, 9, 13), (2, 6, 10, 14), (3, 7, 11, 15))
# this is the module number that corresponds to each zone
pa_zone_sink_indexes = (0, 1, 2, 3)
pa_max_volume = 65536

sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
# make sure we can't get stuck, but also give PA time to respond
sock.settimeout(1)

def ConnectToPA(socket_path):
    sock.connect(socket_path)
    
def DisconnectFromPA():
    sock.close()

def ReadPAResponse():
    resp = ""
    while 1 == 1:
        resp_piece = ""
        resp_piece = sock.recv(4096)
                        
        resp += resp_piece
        
        # check for the prompt string from PA
        if resp_piece.find(pa_prompt_string) >= 0:
            resp = resp.replace(pa_prompt_string, "")
            break
    
    return resp

def SendPACommand(command):
    sock.send(command)
    sock.send("\n")
    return ReadPAResponse()

def GetVolumeInfo():
    list_sinks_string = SendPACommand(pa_list_sinks_string)
    sinks_list = list_sinks_string.split(pa_index_string)
    
    sink_list = list()
    for sink_info in sinks_list:
        sink = sink_info.split("\n")
        if sink[0].isdigit():
            sink_number = int(sink[0])
            # make sure the input is in the range we want
            if sink_number >= pa_first_sink_number and sink_number < pa_first_sink_number + total_zones:
                sink_list.append(sink_info)

    volume_list = list()
    for sink in sink_list:
        # the module index is the first line of the zone info
        sink_index = -1        
        if sink.find("\n") >= 0:
            index = sink[:sink.find("\n")]
            if index.isdigit():
                sink_index = int(index)

        # we look for the unmuted input, that tells us the player that is being listened to
        if sink.find(pa_volume_string) >= 0:
            volume_value_position = sink.find(pa_volume_string) + len(pa_volume_string)
            volume_string = sink[volume_value_position:]
            if volume_string.find(">") >= 0:
                volume_string = volume_string[:volume_string.find(">")]
                volume_string = volume_string[3:]
                if volume_string.find("%") >= 0:
                    volume_string = volume_string[:volume_string.find("%")]
                    if sink_index != -1:
                        volume_list.append(volume_string)

    return volume_list
    

def GetZoneInfo():
    sink_input_string = SendPACommand(pa_list_sink_inputs_string)
    sink_input_list = sink_input_string.split(pa_index_string)
    
    input_list = list()
    for sink_input in sink_input_list:
        input = sink_input.split("\n")
        if input[0].isdigit():
            input_number = int(input[0])
            # make sure the input is in the range we want
            if input_number >= pa_first_input_number and input_number < pa_first_input_number + (total_zones * total_zones):
                input_list.append(sink_input)

    unmuted_list = list()
    for input in input_list:
        # the module index is the first line of the zone info
        input_index = -1        
        if input.find("\n") >= 0:
            index = input[:input.find("\n")]
            if index.isdigit():
                input_index = int(index)

        # we look for the unmuted input, that tells us the player that is being listened to
        if input.find(pa_mute_string) >= 0:
            mute_value_position = input.find(pa_mute_string) + len(pa_mute_string)
            if input[mute_value_position: mute_value_position + 1] == "0":
                # this is the unmuted input
                if input_index != -1:
                    unmuted_list.append([(input_index % total_zones), int(input_index / total_zones)])
        
    unmuted_list.sort()

    return unmuted_list

def SetVolume(zone_number, volume):
    if zone_number > 0 and zone_number <= total_zones:
        if volume >= 0 and volume <= 100:
            volume = volume * pa_max_volume / 100
            SendPACommand("set-sink-volume " + str(pa_zone_sink_indexes[zone_number - 1]) + " " + str(volume))
            return GetVolumeInfo()[zone_number - 1]
        
    return "Fail: incorrect value(s) supplied"

def ConfigZone(zone_number, player_number):
    # mute everything for that zone
    for input in pa_zone_inputs[zone_number]:
        print "set-sink-input-mute " + str(input) + " 1"
        SendPACommand("set-sink-input-mute " + str(input) + " 1")

    # now unmute the proper input
    unmuted_index = pa_zone_inputs[zone_number][player_number]
    print "set-sink-input-mute " + str(unmuted_index) + " 0"
    SendPACommand("set-sink-input-mute " + str(unmuted_index) + " 0")
        
    return GetZoneInfo()
    
def GetConfig():
    return GetZoneInfo()

def SetDefaultConfig():
    for i in range(0, total_zones):
        ConfigZone(i, i)
        
    return GetZoneInfo()

def PrintUsageInstructions():
    print "Audio Matrix Switch Control parameter options:"
    print "\tconfig_zone <zone_number> <player_number>"
    print "\tget_config"
    print "\tset_default_config"

def Main():    
    # check for our command line arguments
    if len(sys.argv) > 1:
        if sys.argv[1] == "config_zone":
            if len(sys.argv) == 4:
                zone = sys.argv[2]
                player = sys.argv[3]
                
                if zone.isdigit() and player.isdigit() and int(zone) > 0 and int(zone) <= total_zones and int(player) > 0 and int(player) <= total_zones:
                    print ConfigZone(int(zone), int(player))
                else:
                    print "Invalid zone or player number. Only 1-" + str(total_zones) + " are valid values." 
                    return
            else:
                PrintUsageInstructions()                    
                return
        elif sys.argv[1] == "set_default_config":
            print SetDefaultConfig()
        elif sys.argv[1] == "get_config":
            print GetConfig()
        elif sys.argv[1] == "get_volume":
            print GetVolumeInfo()
        elif sys.argv[1] == "set_volume":
            if len(sys.argv) == 4:
                zone = sys.argv[2]
                volume = sys.argv[3]
                
                if zone.isdigit() and volume.isdigit() and int(zone) > 0 and int(zone) <= total_zones and int(volume) >= 0 and int(volume) <= 100:
                    print SetVolume(int(zone), int(volume))
                else:
                    print "Invalid zone or player number. Only 1-" + str(total_zones) + " are valid values." 
                    print "Or possibly invalid volume level. 0-100 are valid values"
                    return
        else:
            PrintUsageInstructions()
            return
    else:
        PrintUsageInstructions()
        return


ConnectToPA(default_socket_path)
ReadPAResponse()

Main()

DisconnectFromPA()
